{-
Copyright 2009 Christian Sternagel, René Thiemann, Sarah Winkler, Harald Zankl

This file is part of IsaFoR/CeTA.

IsaFoR/CeTA is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

IsaFoR/CeTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with IsaFoR/CeTA. If not, see <http://www.gnu.org/licenses/>.
-}
module List where {

import Nat;
import HOL;

hd :: forall a. [a] -> a;
hd (x : xs) = x;

foldla :: forall a b. (a -> b -> a) -> a -> [b] -> a;
foldla f a (x : xs) = foldla f (f a x) xs;
foldla f a [] = a;

mapa :: forall b a. (b -> a) -> [b] -> [a];
mapa f (x : xs) = f x : mapa f xs;
mapa f [] = [];

nth :: forall a. [a] -> Nat -> a;
nth (x : xs) n = (case n of {
                   Zero_nat -> x;
                   Suc a -> nth xs a;
                 });

list_case :: forall t a. t -> (a -> [a] -> t) -> [a] -> t;
list_case f1 f2 [] = f1;
list_case f1 f2 (a : list) = f2 a list;

zipa :: forall a b. [a] -> [b] -> [(a, b)];
zipa xs (y : ys) = (case xs of {
                     [] -> [];
                     z : zs -> (z, y) : zipa zs ys;
                   });
zipa xs [] = [];

size_list :: forall a. [a] -> Nat;
size_list (a : list) = plus_nat (size_list list) (Suc Zero_nat);
size_list [] = Zero_nat;

append :: forall a. [a] -> [a] -> [a];
append (x : xs) ys = x : append xs ys;
append [] ys = ys;

foldra :: forall b a. (b -> a -> a) -> [b] -> a -> a;
foldra f (x : xs) a = f x (foldra f xs a);
foldra f [] a = a;

dropa :: forall a. Nat -> [a] -> [a];
dropa n (x : xs) = (case n of {
                     Zero_nat -> x : xs;
                     Suc m -> dropa m xs;
                   });
dropa n [] = [];

takea :: forall a. Nat -> [a] -> [a];
takea n (x : xs) = (case n of {
                     Zero_nat -> [];
                     Suc m -> x : takea m xs;
                   });
takea n [] = [];

member :: forall a. (Eq a) => a -> [a] -> Bool;
member x (y : ys) = (if eqop y x then True else member x ys);
member x [] = False;

data Nibble = NibbleF | NibbleE | NibbleD | NibbleC | NibbleB | NibbleA
  | Nibble9 | Nibble8 | Nibble7 | Nibble6 | Nibble5 | Nibble4 | Nibble3
  | Nibble2 | Nibble1 | Nibble0 deriving (Read, Show);

concata :: forall a. [[a]] -> [a];
concata (x : xs) = append x (concata xs);
concata [] = [];

filtera :: forall a. (a -> Bool) -> [a] -> [a];
filtera p (x : xs) = (if p x then x : filtera p xs else filtera p xs);
filtera p [] = [];

remdups :: forall a. (Eq a) => [a] -> [a];
remdups (x : xs) = (if member x xs then remdups xs else x : remdups xs);
remdups [] = [];

eq_nibble :: Nibble -> Nibble -> Bool;
eq_nibble Nibble0 Nibble0 = True;
eq_nibble Nibble1 Nibble1 = True;
eq_nibble Nibble2 Nibble2 = True;
eq_nibble Nibble3 Nibble3 = True;
eq_nibble Nibble4 Nibble4 = True;
eq_nibble Nibble5 Nibble5 = True;
eq_nibble Nibble6 Nibble6 = True;
eq_nibble Nibble7 Nibble7 = True;
eq_nibble Nibble8 Nibble8 = True;
eq_nibble Nibble9 Nibble9 = True;
eq_nibble NibbleA NibbleA = True;
eq_nibble NibbleB NibbleB = True;
eq_nibble NibbleC NibbleC = True;
eq_nibble NibbleD NibbleD = True;
eq_nibble NibbleE NibbleE = True;
eq_nibble NibbleF NibbleF = True;
eq_nibble Nibble0 Nibble1 = False;
eq_nibble Nibble0 Nibble2 = False;
eq_nibble Nibble0 Nibble3 = False;
eq_nibble Nibble0 Nibble4 = False;
eq_nibble Nibble0 Nibble5 = False;
eq_nibble Nibble0 Nibble6 = False;
eq_nibble Nibble0 Nibble7 = False;
eq_nibble Nibble0 Nibble8 = False;
eq_nibble Nibble0 Nibble9 = False;
eq_nibble Nibble0 NibbleA = False;
eq_nibble Nibble0 NibbleB = False;
eq_nibble Nibble0 NibbleC = False;
eq_nibble Nibble0 NibbleD = False;
eq_nibble Nibble0 NibbleE = False;
eq_nibble Nibble0 NibbleF = False;
eq_nibble Nibble1 Nibble2 = False;
eq_nibble Nibble1 Nibble3 = False;
eq_nibble Nibble1 Nibble4 = False;
eq_nibble Nibble1 Nibble5 = False;
eq_nibble Nibble1 Nibble6 = False;
eq_nibble Nibble1 Nibble7 = False;
eq_nibble Nibble1 Nibble8 = False;
eq_nibble Nibble1 Nibble9 = False;
eq_nibble Nibble1 NibbleA = False;
eq_nibble Nibble1 NibbleB = False;
eq_nibble Nibble1 NibbleC = False;
eq_nibble Nibble1 NibbleD = False;
eq_nibble Nibble1 NibbleE = False;
eq_nibble Nibble1 NibbleF = False;
eq_nibble Nibble2 Nibble3 = False;
eq_nibble Nibble2 Nibble4 = False;
eq_nibble Nibble2 Nibble5 = False;
eq_nibble Nibble2 Nibble6 = False;
eq_nibble Nibble2 Nibble7 = False;
eq_nibble Nibble2 Nibble8 = False;
eq_nibble Nibble2 Nibble9 = False;
eq_nibble Nibble2 NibbleA = False;
eq_nibble Nibble2 NibbleB = False;
eq_nibble Nibble2 NibbleC = False;
eq_nibble Nibble2 NibbleD = False;
eq_nibble Nibble2 NibbleE = False;
eq_nibble Nibble2 NibbleF = False;
eq_nibble Nibble3 Nibble4 = False;
eq_nibble Nibble3 Nibble5 = False;
eq_nibble Nibble3 Nibble6 = False;
eq_nibble Nibble3 Nibble7 = False;
eq_nibble Nibble3 Nibble8 = False;
eq_nibble Nibble3 Nibble9 = False;
eq_nibble Nibble3 NibbleA = False;
eq_nibble Nibble3 NibbleB = False;
eq_nibble Nibble3 NibbleC = False;
eq_nibble Nibble3 NibbleD = False;
eq_nibble Nibble3 NibbleE = False;
eq_nibble Nibble3 NibbleF = False;
eq_nibble Nibble4 Nibble5 = False;
eq_nibble Nibble4 Nibble6 = False;
eq_nibble Nibble4 Nibble7 = False;
eq_nibble Nibble4 Nibble8 = False;
eq_nibble Nibble4 Nibble9 = False;
eq_nibble Nibble4 NibbleA = False;
eq_nibble Nibble4 NibbleB = False;
eq_nibble Nibble4 NibbleC = False;
eq_nibble Nibble4 NibbleD = False;
eq_nibble Nibble4 NibbleE = False;
eq_nibble Nibble4 NibbleF = False;
eq_nibble Nibble5 Nibble6 = False;
eq_nibble Nibble5 Nibble7 = False;
eq_nibble Nibble5 Nibble8 = False;
eq_nibble Nibble5 Nibble9 = False;
eq_nibble Nibble5 NibbleA = False;
eq_nibble Nibble5 NibbleB = False;
eq_nibble Nibble5 NibbleC = False;
eq_nibble Nibble5 NibbleD = False;
eq_nibble Nibble5 NibbleE = False;
eq_nibble Nibble5 NibbleF = False;
eq_nibble Nibble6 Nibble7 = False;
eq_nibble Nibble6 Nibble8 = False;
eq_nibble Nibble6 Nibble9 = False;
eq_nibble Nibble6 NibbleA = False;
eq_nibble Nibble6 NibbleB = False;
eq_nibble Nibble6 NibbleC = False;
eq_nibble Nibble6 NibbleD = False;
eq_nibble Nibble6 NibbleE = False;
eq_nibble Nibble6 NibbleF = False;
eq_nibble Nibble7 Nibble8 = False;
eq_nibble Nibble7 Nibble9 = False;
eq_nibble Nibble7 NibbleA = False;
eq_nibble Nibble7 NibbleB = False;
eq_nibble Nibble7 NibbleC = False;
eq_nibble Nibble7 NibbleD = False;
eq_nibble Nibble7 NibbleE = False;
eq_nibble Nibble7 NibbleF = False;
eq_nibble Nibble8 Nibble9 = False;
eq_nibble Nibble8 NibbleA = False;
eq_nibble Nibble8 NibbleB = False;
eq_nibble Nibble8 NibbleC = False;
eq_nibble Nibble8 NibbleD = False;
eq_nibble Nibble8 NibbleE = False;
eq_nibble Nibble8 NibbleF = False;
eq_nibble Nibble9 NibbleA = False;
eq_nibble Nibble9 NibbleB = False;
eq_nibble Nibble9 NibbleC = False;
eq_nibble Nibble9 NibbleD = False;
eq_nibble Nibble9 NibbleE = False;
eq_nibble Nibble9 NibbleF = False;
eq_nibble NibbleA NibbleB = False;
eq_nibble NibbleA NibbleC = False;
eq_nibble NibbleA NibbleD = False;
eq_nibble NibbleA NibbleE = False;
eq_nibble NibbleA NibbleF = False;
eq_nibble NibbleB NibbleC = False;
eq_nibble NibbleB NibbleD = False;
eq_nibble NibbleB NibbleE = False;
eq_nibble NibbleB NibbleF = False;
eq_nibble NibbleC NibbleD = False;
eq_nibble NibbleC NibbleE = False;
eq_nibble NibbleC NibbleF = False;
eq_nibble NibbleD NibbleE = False;
eq_nibble NibbleD NibbleF = False;
eq_nibble NibbleE NibbleF = False;
eq_nibble Nibble1 Nibble0 = False;
eq_nibble Nibble2 Nibble0 = False;
eq_nibble Nibble3 Nibble0 = False;
eq_nibble Nibble4 Nibble0 = False;
eq_nibble Nibble5 Nibble0 = False;
eq_nibble Nibble6 Nibble0 = False;
eq_nibble Nibble7 Nibble0 = False;
eq_nibble Nibble8 Nibble0 = False;
eq_nibble Nibble9 Nibble0 = False;
eq_nibble NibbleA Nibble0 = False;
eq_nibble NibbleB Nibble0 = False;
eq_nibble NibbleC Nibble0 = False;
eq_nibble NibbleD Nibble0 = False;
eq_nibble NibbleE Nibble0 = False;
eq_nibble NibbleF Nibble0 = False;
eq_nibble Nibble2 Nibble1 = False;
eq_nibble Nibble3 Nibble1 = False;
eq_nibble Nibble4 Nibble1 = False;
eq_nibble Nibble5 Nibble1 = False;
eq_nibble Nibble6 Nibble1 = False;
eq_nibble Nibble7 Nibble1 = False;
eq_nibble Nibble8 Nibble1 = False;
eq_nibble Nibble9 Nibble1 = False;
eq_nibble NibbleA Nibble1 = False;
eq_nibble NibbleB Nibble1 = False;
eq_nibble NibbleC Nibble1 = False;
eq_nibble NibbleD Nibble1 = False;
eq_nibble NibbleE Nibble1 = False;
eq_nibble NibbleF Nibble1 = False;
eq_nibble Nibble3 Nibble2 = False;
eq_nibble Nibble4 Nibble2 = False;
eq_nibble Nibble5 Nibble2 = False;
eq_nibble Nibble6 Nibble2 = False;
eq_nibble Nibble7 Nibble2 = False;
eq_nibble Nibble8 Nibble2 = False;
eq_nibble Nibble9 Nibble2 = False;
eq_nibble NibbleA Nibble2 = False;
eq_nibble NibbleB Nibble2 = False;
eq_nibble NibbleC Nibble2 = False;
eq_nibble NibbleD Nibble2 = False;
eq_nibble NibbleE Nibble2 = False;
eq_nibble NibbleF Nibble2 = False;
eq_nibble Nibble4 Nibble3 = False;
eq_nibble Nibble5 Nibble3 = False;
eq_nibble Nibble6 Nibble3 = False;
eq_nibble Nibble7 Nibble3 = False;
eq_nibble Nibble8 Nibble3 = False;
eq_nibble Nibble9 Nibble3 = False;
eq_nibble NibbleA Nibble3 = False;
eq_nibble NibbleB Nibble3 = False;
eq_nibble NibbleC Nibble3 = False;
eq_nibble NibbleD Nibble3 = False;
eq_nibble NibbleE Nibble3 = False;
eq_nibble NibbleF Nibble3 = False;
eq_nibble Nibble5 Nibble4 = False;
eq_nibble Nibble6 Nibble4 = False;
eq_nibble Nibble7 Nibble4 = False;
eq_nibble Nibble8 Nibble4 = False;
eq_nibble Nibble9 Nibble4 = False;
eq_nibble NibbleA Nibble4 = False;
eq_nibble NibbleB Nibble4 = False;
eq_nibble NibbleC Nibble4 = False;
eq_nibble NibbleD Nibble4 = False;
eq_nibble NibbleE Nibble4 = False;
eq_nibble NibbleF Nibble4 = False;
eq_nibble Nibble6 Nibble5 = False;
eq_nibble Nibble7 Nibble5 = False;
eq_nibble Nibble8 Nibble5 = False;
eq_nibble Nibble9 Nibble5 = False;
eq_nibble NibbleA Nibble5 = False;
eq_nibble NibbleB Nibble5 = False;
eq_nibble NibbleC Nibble5 = False;
eq_nibble NibbleD Nibble5 = False;
eq_nibble NibbleE Nibble5 = False;
eq_nibble NibbleF Nibble5 = False;
eq_nibble Nibble7 Nibble6 = False;
eq_nibble Nibble8 Nibble6 = False;
eq_nibble Nibble9 Nibble6 = False;
eq_nibble NibbleA Nibble6 = False;
eq_nibble NibbleB Nibble6 = False;
eq_nibble NibbleC Nibble6 = False;
eq_nibble NibbleD Nibble6 = False;
eq_nibble NibbleE Nibble6 = False;
eq_nibble NibbleF Nibble6 = False;
eq_nibble Nibble8 Nibble7 = False;
eq_nibble Nibble9 Nibble7 = False;
eq_nibble NibbleA Nibble7 = False;
eq_nibble NibbleB Nibble7 = False;
eq_nibble NibbleC Nibble7 = False;
eq_nibble NibbleD Nibble7 = False;
eq_nibble NibbleE Nibble7 = False;
eq_nibble NibbleF Nibble7 = False;
eq_nibble Nibble9 Nibble8 = False;
eq_nibble NibbleA Nibble8 = False;
eq_nibble NibbleB Nibble8 = False;
eq_nibble NibbleC Nibble8 = False;
eq_nibble NibbleD Nibble8 = False;
eq_nibble NibbleE Nibble8 = False;
eq_nibble NibbleF Nibble8 = False;
eq_nibble NibbleA Nibble9 = False;
eq_nibble NibbleB Nibble9 = False;
eq_nibble NibbleC Nibble9 = False;
eq_nibble NibbleD Nibble9 = False;
eq_nibble NibbleE Nibble9 = False;
eq_nibble NibbleF Nibble9 = False;
eq_nibble NibbleB NibbleA = False;
eq_nibble NibbleC NibbleA = False;
eq_nibble NibbleD NibbleA = False;
eq_nibble NibbleE NibbleA = False;
eq_nibble NibbleF NibbleA = False;
eq_nibble NibbleC NibbleB = False;
eq_nibble NibbleD NibbleB = False;
eq_nibble NibbleE NibbleB = False;
eq_nibble NibbleF NibbleB = False;
eq_nibble NibbleD NibbleC = False;
eq_nibble NibbleE NibbleC = False;
eq_nibble NibbleF NibbleC = False;
eq_nibble NibbleE NibbleD = False;
eq_nibble NibbleF NibbleD = False;
eq_nibble NibbleF NibbleE = False;

}
