{-
Copyright 2009 Christian Sternagel, René Thiemann, Sarah Winkler, Harald Zankl

This file is part of IsaFoR/CeTA.

IsaFoR/CeTA is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

IsaFoR/CeTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with IsaFoR/CeTA. If not, see <http://www.gnu.org/licenses/>.
-}
module ProofChecker where {

import qualified Loop;
import qualified Term;
import qualified ErrorMonad;
import qualified DepGraph;
import qualified List;
import qualified Nat;
import qualified Polynomial;
import qualified Trs;
import qualified Show;
import qualified Util;
import qualified DpTransImpl;

data DPProof a b =
  DepGraphProc
    [(Maybe (ProofChecker.DPProof a b), [(Term.Term a b, Term.Term a b)])]
  | RedPairProc (Polynomial.RedPair a) [(Term.Term a b, Term.Term a b)]
      (ProofChecker.DPProof a b)
  | PisEmpty deriving (Read, Show);

data TRSProof a b =
  Loop ([(Term.Term a b)], (Term.Ctxt a b, [(Term.Term a b, Term.Term a b)]))
  | DPTrans [(Term.Term (Trs.Shp a) b, Term.Term (Trs.Shp a) b)]
      (ProofChecker.DPProof (Trs.Shp a) b)
  deriving (Read, Show);

indent :: [[Char]] -> [[Char]];
indent xs = "\n" : List.mapa (List.append "   ") xs;

checkLoop ::
  forall a b.
    (Eq a,
      Eq b) => [(Term.Term a b, Term.Term a b)] ->
                 ([(Term.Term a b)],
                   (Term.Ctxt a b, [(Term.Term a b, Term.Term a b)]))
                   -> Util.Eithera [[Char]] ();
checkLoop r (ts, (c, sub)) =
  (case Loop.subst_safe [] sub (Term.Sub Term.Var) of {
    Nothing -> ErrorMonad.errora ["substitution not unique"];
    Just sigma ->
      ErrorMonad.checkB (Loop.check_loop (ts, (c, sigma)) r)
        ["Loop not certified"];
  });

checkDPProof ::
  forall a b.
    (Eq a, Show.Sshow a, Eq b,
      Show.Sshow b) => [(Term.Term a b, Term.Term a b)] ->
                         [(Term.Term a b, Term.Term a b)] ->
                           ProofChecker.DPProof a b -> Util.Eithera [[Char]] ();
checkDPProof p r (ProofChecker.DepGraphProc edpts) =
  ErrorMonad.bind
    (ErrorMonad.catcha
      (DepGraph.checkDepGraphProc p
        (List.mapa
          (\ x ->
            ((case fst x of {
               Nothing -> False;
               Just xa -> True;
             }),
              snd x))
          edpts)
        r)
      (\ x ->
        ErrorMonad.errora
          ("error while trying to perform SCC-decomposition  on " :
            List.append (Trs.showDPP p r) x)))
    (\ x ->
      ErrorMonad.catcha
        (ErrorMonad.catcha
          (ErrorMonad.checkAllMI
            (\ rp ->
              (case fst rp of {
                Nothing -> ErrorMonad.returna ();
                Just a -> ProofChecker.checkDPProof (snd rp) r a;
              }))
            edpts)
          (\ xa -> ErrorMonad.errora (snd xa)))
        (\ xa ->
          ErrorMonad.errora
            ("error below the dependency graph processor" :
              ProofChecker.indent xa)));
checkDPProof p r (ProofChecker.RedPairProc redp rP prf) =
  ErrorMonad.bind
    (ErrorMonad.catcha (Polynomial.createRedPairProc redp p rP r)
      (\ x ->
        ErrorMonad.errora
          (List.append (Trs.showDPP p r)
            ("error when applying the reduction pair processor to obtain" :
              List.append (Trs.showP rP) x))))
    (\ x ->
      ErrorMonad.catcha (ProofChecker.checkDPProof rP r prf)
        (\ xa ->
          ErrorMonad.errora
            ("error below the reduction pair processor" :
              ProofChecker.indent xa)));
checkDPProof p r ProofChecker.PisEmpty =
  (case p of {
    [] -> ErrorMonad.returna ();
    x : xa ->
      ErrorMonad.errora
        ("P is not empty in the following DP-problem " : Trs.showDPP p r);
  });

checkTRSProof ::
  forall a b.
    (Eq a, Show.Sshow a, Eq b,
      Show.Sshow b) => [(Term.Term a b, Term.Term a b)] ->
                         ProofChecker.TRSProof a b -> Util.Eithera [[Char]] ();
checkTRSProof r (ProofChecker.Loop l) = ProofChecker.checkLoop r l;
checkTRSProof r (ProofChecker.DPTrans p prf) =
  ErrorMonad.bind
    (ErrorMonad.catcha (DpTransImpl.checkDPs r p)
      (\ x ->
        ErrorMonad.errora
          ("error when switching from the TRS" :
            List.append (Trs.showTRS r)
              ("to the initial DP problem" :
                List.append (Trs.showDPP p (DpTransImpl.computeID r)) x))))
    (\ x ->
      ErrorMonad.catcha
        (ProofChecker.checkDPProof p (DpTransImpl.computeID r) prf)
        (\ xa ->
          ErrorMonad.errora
            ("error below switch to dependency pairs" :
              ProofChecker.indent xa)));

}
