{-
Copyright 2009 Christian Sternagel, René Thiemann, Sarah Winkler, Harald Zankl

This file is part of IsaFoR/CeTA.

IsaFoR/CeTA is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

IsaFoR/CeTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with IsaFoR/CeTA. If not, see <http://www.gnu.org/licenses/>.
-}
module Term where {

import qualified Nat;
import qualified List;
import qualified Product_Type;
import qualified HOL;
import qualified Position;
import qualified Show;

data Term a b = Fun a [(Term.Term a b)] | Var b deriving (Read, Show);

newtype Sub a b = Sub (b -> Term.Term a b);

sz :: forall a b. Term.Term a b -> Nat.Nat;
sz (Term.Fun f ts) =
  Nat.plus_nat Nat.one_nat
    (List.foldla (\ a x -> Nat.plus_nat a (Term.sz x)) Nat.Zero_nat ts);
sz (Term.Var x) = Nat.one_nat;

poss_list :: forall a b. Term.Term a b -> [Position.Pos];
poss_list (Term.Fun f ts) =
  let {
    a = List.foldra
          (\ t (a @ (i, ps)) ->
            (Nat.minus_nat i Nat.one_nat,
              List.append ps (List.mapa (Position.PCons i) (Term.poss_list t))))
          ts (Nat.minus_nat (List.size_list ts) Nat.one_nat, []);
    (i, aa) = a;
  } in Position.Empty : aa;
poss_list (Term.Var x) = [Position.Empty];

data Ctxt a b = More a [(Term.Term a b)] (Term.Ctxt a b) [(Term.Term a b)]
  | Hole deriving (Read, Show);

sub_apply :: forall a b. Term.Term a b -> Term.Sub a b -> Term.Term a b;
sub_apply (Term.Fun f ss) sigma =
  Term.Fun f (List.mapa (\ t -> Term.sub_apply t sigma) ss);
sub_apply (Term.Var x) (Term.Sub sigma) = sigma x;

ctxt_apply :: forall a b. Term.Ctxt a b -> Term.Term a b -> Term.Term a b;
ctxt_apply (Term.More f ss1 c ss2) s =
  Term.Fun f (List.append ss1 (Term.ctxt_apply c s : ss2));
ctxt_apply Term.Hole s = s;

eq_term :: forall f v. (Eq f, Eq v) => Term.Term f v -> Term.Term f v -> Bool;
eq_term (Term.Var v) (Term.Var v') = v == v';
eq_term (Term.Fun f list) (Term.Fun f' list') = f == f' && list == list';
eq_term (Term.Var a) (Term.Fun b c) = False;
eq_term (Term.Fun b c) (Term.Var a) = False;

instance (Eq a, Eq b) => Eq (Term.Term a b) where {
  (==) = Term.eq_term;
};

subt_at :: forall a b. Term.Term a b -> Position.Pos -> Term.Term a b;
subt_at (Term.Fun f ss) (Position.PCons i p) = Term.subt_at (List.nth ss i) p;
subt_at s Position.Empty = s;

sub_case :: forall v f t. ((v -> Term.Term f v) -> t) -> Term.Sub f v -> t;
sub_case f1 (Term.Sub fun) = f1 fun;

sshow_term ::
  forall c d. (Show.Sshow c, Show.Sshow d) => Term.Term c d -> [Char];
sshow_term (Term.Fun f ts) =
  List.append (Show.sshow f)
    (case ts of {
      [] -> Show.sEmpty;
      s : ss ->
        List.append "("
          (List.append
            (List.append (Term.sshow_term s)
              (List.concata
                (List.mapa (\ x -> List.concata [",", Term.sshow_term x]) ss)))
            ")");
    });
sshow_term (Term.Var x) = Show.sshow x;

isVar :: forall f v. Term.Term f v -> Bool;
isVar (Term.Fun uu uv) = False;
isVar (Term.Var x) = True;

instance (Show.Sshow a, Show.Sshow b) => Show.Sshow (Term.Term a b) where {
  sshow = Term.sshow_term;
};

}
